"""Example script for creating SBML repressilator with neuroml.

https://neuroml.org/
https://docs.neuroml.org/Userdocs/CreatingNeuroMLModels.html

https://pyneuroml.readthedocs.io/en/latest/pyneuroml.html#module-pyneuroml.pynml
https://docs.biosimulators.org/Biosimulators_pyNeuroML/installation.html
https://github.com/NeuroML/Documentation/blob/master/source/Userdocs/NML2_examples/SingleNeuron.ipynb
"""
